require "budgetsms/version"
require 'yaml'
require 'erb'
require 'net/http'
require 'uri'

class BudgetSMS
  class << self
    API_ENDPOINTS = {
      "TEST_SEND_SMS_URL" => "https://api.budgetsms.net/testsms/",
      "SEND_SMS_URL" => "https://api.budgetsms.net/sendsms/",
      "CHECK_CREDIT_URL" => "https://api.budgetsms.net/checkcredit/"
    }.freeze

    API_ERRORS = {
      "1001" => "Not enough credits to send messages",
      "1002" => "Identification failed. Wrong credentials",
      "1003" => "Account not active, contact BudgetSMS",
      "1004" =>	"This IP address is not added to this account. No access to the API",
      "1005" =>	"No handle provided",
      "1006" =>	"No UserID provided",
      "1007" =>	"No Username provided",
      "2001" =>	"SMS message text is empty",
      "2002" =>	"SMS numeric senderid can be max. 16 numbers",
      "2003" =>	"SMS alphanumeric sender can be max. 11 characters",
      "2004" =>	"SMS senderid is empty or invalid",
      "2005" =>	"Destination number is too short",
      "2006" =>	"Destination is not numeric",
      "2007" =>	"Destination is empty",
      "2008" =>	"SMS text is not OK (check encoding?)",
      "2009" =>	"Parameter issue (check all mandatory parameters, encoding, etc.)",
      "2010" =>	"Destination number is invalidly formatted",
      "2011" =>	"Destination is invalid",
      "2012" =>	"SMS message text is too long",
      "2013" =>	"SMS message is invalid",
      "2014" =>	"SMS CustomID is used before",
      "2015" => "Charset problem",
      "2016" =>	"Invalid UTF-8 encoding",
      "2017" =>	"Invalid SMSid",
      "3001" =>	"No route to destination. Contact BudgetSMS for possible solutions",
      "3002" =>	"No routes are setup. Contact BudgetSMS for a route setup",
      "3003" =>	"Invalid destination. Check international mobile number formatting",
      "4001" =>	"System error, related to customID",
      "4002" =>	"System error, temporary issue. Try resubmitting in 2 to 3 minutes",
      "4003" =>	"System error, temporary issue.",
      "4004" =>	"System error, temporary issue. Contact BudgetSMS",
      "5001" =>	"Send error, Contact BudgetSMS with the send details",
      "5002" =>	"Wrong SMS type",
      "5003" =>	"Wrong operator",
      "6001" =>	"Unknown error",
      "7001" =>	"No HLR provider present, Contact BudgetSMS.",
      "7002" =>	"Unexpected results from HLR provider",
      "7003" =>	"Bad number format",
      "7901" =>	"Unexpected error. Contact BudgetSMS",
      "7902" =>	"HLR provider error. Contact BudgetSMS",
      "7903" =>	"HLR provider error. Contact BudgetSMS"
    }.freeze

    attr_accessor :to, :msg

    def send_sms to, msg
      @to = to
      @msg = msg
      response = get_sms_response

      if response.kind_of? Net::HTTPSuccess
        parse_success_sms response
      else
        message = "BudgetSMS API request failed: #{response.code} #{response.message}"
        logger.fatal message

        raise StandardError.new(message)
      end
    end

    def check_credit
      response = get_check_credit_response
      status, credit = response.body.split(' ')

      if status == 'OK'
        logger.debug "BudgetSMS STATUS:#{status} CREDIT:#{credit}"
        credit.to_f
      else
        rise_api_error response
      end
    end

    private

    def parse_success_sms response
      status, smsid, cost, n, credit = response.body.split(' ')

      if status == 'OK'
        logger.debug "BudgetSMS STATUS:#{status} TO:#{to}, MSG:#{msg}, SMSID:#{smsid}"
        logger.debug "BudgetSMS COST:#{cost}, CREDIT:#{credit}"
      else
        rise_api_error response
      end
    end

    def logger
      logger = Logger.new(STDOUT)
    end

    def get_check_credit_response
      config = load_config
      uri = URI.parse(API_ENDPOINTS["CHECK_CREDIT_URL"])
      params = {
        :username => config['BUDGETSMS_USERNAME'],
        :userid => config['BUDGETSMS_USERID'],
        :handle => config['BUDGETSMS_HANDLE']
      }

      uri.query = URI.encode_www_form(params)
      Net::HTTP.get_response(uri)
    end

    def get_sms_response
      config = load_config
      send_sms_url = Rails.env == 'production' ? API_ENDPOINTS["SEND_SMS_URL"] : API_ENDPOINTS["TEST_SEND_SMS_URL"]
      uri = URI.parse(send_sms_url)
      params = {
        :username => config['BUDGETSMS_USERNAME'],
        :userid => config['BUDGETSMS_USERID'],
        :handle => config['BUDGETSMS_HANDLE'],
        :from => config['BUDGETSMS_FROM'],
        :to => to,
        :price => 1,
        :credit => 1,
        :msg => msg
      }

      uri.query = URI.encode_www_form(params)
      Net::HTTP.get_response(uri)
    end

    def rise_api_error response
      type, error_code = response.body.split(' ')
      message = "BudgetSMS API returned not-OK: #{API_ERRORS[error_code]}"
      logger.fatal message

      raise StandardError.new(message)
    end

    def load_config
      YAML.load(ERB.new(File.read(Rails.root.join('config/budgetsms.yml'))).result)
    end
  end
end
